#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>

#include "tools.hpp"
#include "rbrcar.hpp"


using namespace std;

string rbrdir;
string dattooldir;
int tasks = 1;
int task = 1;

void importSlot (string slotName, string carDir);
void exportCar (string carDir, string slotName);

void cleanSlot (int slot);
string getPhysicsSlotDir (int slot);

void exportGraphics (string car, int slot);
void exportPhysics (string car, int slot);

void patchFiles (RBRCar car, int slot);
    void patchCarsCarIni (RBRCar car, int slot);
     int locateCarsCarIniLine (int slot);
    void patchMiscCarinfoIni (RBRCar car, int slot);
     int locateMiscCarinfoIniIniLine (int slot);
    void patchShadersIni (RBRCar car, int slot);
     int locateShaderIniLine(string shaderFile);
    void patchCarExes (RBRCar car, int slot);
     int getOffset(int slot, char set, bool sse);
    void patchMiscAiAidrvIniTeam (RBRCar car, int slot);
     int locateMiscAiAidrvIniTeamLine(int slot);
    void patchMiscAiAidrvIniCar (RBRCar car, int slot);
     int locateMiscAiAidrvIniCarLine(int slot);
    void patchSplashImage (RBRCar car, int slot);


void showHelp(bool continuedByDetailedHelp = false)
{
    cout << "RBRCM - Richard Burns Rally Car Manager v" << version << endl;
    cout << "SYNOPSIS" << endl;
    cout << "\trbrcm --import SLOT RBRCMCAR" << endl;
    cout << "\trbrcm --export RBRCMCAR SLOT" << endl;
    cout << "\trbrcm --backup originals" << endl;
    cout << "\trbrcm --restore originals" << endl;
    cout << "\trbrcm --help" << endl;
    cout << endl;
    cout << "COPYRIGHT" << endl;
    cout << "\tCopyright(C)2005 Bruno Gonzalez. (aka STenyaK; stenyak AT gmx DOT net)" << endl;
    cout << "\tThis is free software; see the source for copying  conditions.   There" << endl;
    cout << "\tis NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICU-" << endl;
    cout << "\tLAR PURPOSE." << endl;
    if (!continuedByDetailedHelp)
    {
    cout << endl;
    cout << "Type \"rbrcm --help\" for detailed help on all the commands." << endl;
    }    
}    

void showDetailedHelp()
{
    showHelp(true);
    cout << endl;
    cout << "DESCRIPTION" << endl;
    cout << "\tAllows to manage Richard Burns Rally cars, ideal for use by 3rd. party" << endl;
    cout << "\tmods. There are two possible actions to perform: export and import:" << endl;
    cout << endl << "Press any key to continue." << endl;
    system ("pause >nul");
    cout << endl;
    cout << "\t--import SLOT RBRCMCAR" << endl;
    cout << "\tImport a RBR car to RBRCM format. SLOT can be any from \"slot00\" to" << endl;
    cout << "\t\"slot07\". RBRCMCAR is the name of the directory (under ./cars) where" << endl;
    cout << "\tthe car will be put in RBRCM format." << endl;
    cout << endl;
    cout << "\t--export RBRCMCAR SLOT" << endl;
    cout << "\tExports a RBRCM car to RBR. RBRCMCAR must be an existing car located" << endl;
    cout << "\tin \"./cars/RBRCMCAR\". SLOT can be any from \"slot00\" to \"slot07\"." << endl;
    cout << endl;
    cout << "\t--backup originals" << endl;
    cout << "\tAssuming you have a fresh, default RBR install, it imports the 8 cars" << endl;
    cout << "\tto RBRCM format, with the names \"[original]\"+carName." << endl;
    cout << endl;
    cout << "\t--restore originals" << endl;
    cout << "\tAssuming you have first made a backup, it restores the 8 original cars" << endl;
    cout << "\tto RBR format, in <rbrDir>\\slot00 to slot07." << endl;
    cout << endl;
    cout << "\t--help" << endl;
    cout << "\tShows this detailed help screen." << endl;
}    

void readConfig(string configFile)
{
    FILE * file = fileOpen(configFile, "r");
    const int linesize = 4096;
    char tmpline[linesize];
    if (!fgets (tmpline, linesize, file))
    {
        cout << "Error while parsing \"rbrcm.ini\" config file." << endl;
        cout << "Please check that it's not empty." << endl;
        exit (-2);
    }    
    rbrdir = tmpline;
    rbrdir.resize (rbrdir.length() -1);
    if (!fgets (tmpline, linesize, file))
    {
        cout << "Error while parsing \"rbrcm.ini\" config file." << endl;
        cout << "Please check that there's it has at least two lines." << endl;
        exit (-2);
    }
    dattooldir = tmpline;
    dattooldir.resize (dattooldir.length() -1);
    fclose (file);
}    

/* Returns:
       -3 -> error reading car config file
       -2 -> error reading rbrcm config file
       -1 -> incorrect number of parameters
       1 -> correct initialization
       2 -> correct replacement
*/
int main(int argc, char *argv[])
{
    // read rbrcm config file
    readConfig("rbrcm.ini");
   
    // check what action to perform
    if (argc == 4)
    {
        string command (argv[1]);
        string arg1 (argv[2]);
        string arg2 (argv[3]);
        if (command == "--import")
        {
            // import --> from rbr to rbrcm
            importSlot (arg1, arg2);
            return 2;
        }else
        if (command == "--export")
        {
            // export --> from rbrcm to rbr
            exportCar (arg1, arg2);
            return 2;
        }else{    
            cout << "Invalid arguments!" << endl;
            showHelp();   
            return -1;
        }
    }else if (argc == 3)
    {
        string command (argv[1]);
        string arg1 (argv[2]);
        if (command == "--backup" && arg1 == "originals")
        {
            cout << "===============================================================================" << endl;
            cout << "Making a backup of all 8 original RBR cars..." << endl;
            cout << "===============================================================================" << endl;
            cout << endl << endl;
            importSlot("slot00", "[original]citroenXsara");
            cout << endl << endl;
            importSlot("slot01", "[original]hyundaiAccent");
            cout << endl << endl;
            importSlot("slot02", "[original]mgZr");
            cout << endl << endl;
            importSlot("slot03", "[original]mitsubishiLancerEvoVII");
            cout << endl << endl;
            importSlot("slot04", "[original]peugeot206");
            cout << endl << endl;
            importSlot("slot05", "[original]subaruImpreza2003");
            cout << endl << endl;
            importSlot("slot06", "[original]toyotaCorolla");
            cout << endl << endl;
            importSlot("slot07", "[original]subaruImpreza2000");
            cout << endl << endl;
            cout << "===============================================================================" << endl;
            cout << "Done! Backup of the 8 original cars sucessfully made." << endl;
            cout << "They have been stored with the following RBRCM names:" << endl;
            cout << "\t[original]citroenXsara" << endl;
            cout << "\t[original]hyundaiAccent" << endl;
            cout << "\t[original]mgZr" << endl;
            cout << "\t[original]mitsubishiLancerEvoVII" << endl;
            cout << "\t[original]peugeot206" << endl;
            cout << "\t[original]subaruImpreza2003" << endl;
            cout << "\t[original]toyotaCorolla" << endl;
            cout << "\t[original]subaruImpreza2000" << endl;
            cout << "You can import them as usual, execute \"rbrcm\" for more info." << endl;
            cout << "===============================================================================" << endl;
        }else if (command == "--restore" && arg1 == "originals")
        {
            cout << "===============================================================================" << endl;
            cout << "Restoring all the 8 original RBR cars..." << endl;
            cout << "===============================================================================" << endl;
            cout << endl << endl;
            exportCar ("[original]citroenXsara", "slot00");
            cout << endl << endl;
            exportCar ("[original]hyundaiAccent", "slot01");
            cout << endl << endl;
            exportCar ("[original]mgZr", "slot02");
            cout << endl << endl;
            exportCar ("[original]mitsubishiLancerEvoVII", "slot03");
            cout << endl << endl;
            exportCar ("[original]peugeot206", "slot04");
            cout << endl << endl;
            exportCar ("[original]subaruImpreza2003", "slot05");
            cout << endl << endl;
            exportCar ("[original]toyotaCorolla", "slot06");
            cout << endl << endl;
            exportCar ("[original]subaruImpreza2000", "slot07");
            cout << endl << endl;
            cout << "===============================================================================" << endl;
            cout << "Done! The 8 original cars have been sucessfully restored." << endl;
            cout << "They are now placed in:" << endl;
            cout << "\t" << rbrdir << "\\slot00" << endl;
            cout << "\t" << rbrdir << "\\slot01" << endl;
            cout << "\t" << rbrdir << "\\slot02" << endl;
            cout << "\t" << rbrdir << "\\slot03" << endl;
            cout << "\t" << rbrdir << "\\slot04" << endl;
            cout << "\t" << rbrdir << "\\slot05" << endl;
            cout << "\t" << rbrdir << "\\slot06" << endl;
            cout << "\t" << rbrdir << "\\slot07" << endl;
            cout << "===============================================================================" << endl;
        }else{
            cout << "Invalid arguments!" << endl;
            showHelp();   
            return -1;
        }        
    }else if (argc == 2)
    {
        string command (argv[1]);
        if (command == "--help")
        {
            showDetailedHelp();   
        }else{    
            cout << "Invalid arguments!" << endl;
            showHelp();   
            return -1;
        }
    }else{    
        cout << "Invalid number of arguments!" << endl;
        showHelp();
        return -1;
    }    
    // should never get here...
}


int stoslot (string slotName)
{
    // check slot string length
    if (slotName.length() != 6)
    {
        cout << "Incorrect slot name \"" << slotName << "\"." << endl;
        cout << "Slot syntax: slotXX" << endl;
        cout << "where XX is a number between 00 and 07" << endl;   
        exit (-1);
    }

    // convert to integer
    string slotNumber = slotName.substr(4, 2);
    int slot = stoi (slotNumber);

    // check integer range
    if (slot < 0 || slot > 7)
    {
        cout << "Incorrect slot number \"" << slot << "\"." << endl;
        cout << "Slot syntax: slotXX" << endl;
        cout << "where XX is a number between 00 and 07" << endl;   
        exit (-1);
    }
    
    return (slot);
}

void cleanDir (string dir)
{
    string tmpString ("deltree /Y \"" + dir + "\"");
    system (tmpString.c_str());
    tmpString = "rmdir /s \"" + dir + "\"";
    system (tmpString.c_str());
}    
void createDir (string dir)
{
    string tmpString ("mkdir \"" + dir + "\"");
    cout << tmpString << endl;
    system (tmpString.c_str());
}    

string getGraphicsImportDir(int slot)
{
    int slotLine = locateCarsCarIniLine(slot);
    string filename (rbrdir + "\\cars\\cars.ini");
    string line = freads(filename, slotLine);

    int initialPosition = line.find("\"") + 1;
    int finalPosition = line.find ("\\", initialPosition);
    finalPosition = line.find ("\\", finalPosition + 1);
    int length = finalPosition - initialPosition;

    line = line.substr(initialPosition, length);
    return (line);
}    
void importGraphics (int slot, string carDir)
{
    string slotDir = getGraphicsImportDir(slot);
    string tmpString ("xcopy /e /c /h /r /y /i /q \"" + rbrdir + "\\" + slotDir + "\" cars\\" + carDir + "\\graphics\"");
    system (tmpString.c_str());
}    
void importPhysics (int slot, string carDir)
{
    string fsxDir = getPhysicsSlotDir(slot);
    string tmpString ("xcopy /e /c /h /r /y /i /q \"" + fsxDir + "\" cars\\" + carDir + "\\physics\"");
    system (tmpString.c_str());
}    

void readCarsCarIni (RBRCar & car, int slot)
{
    int replaceLine = locateCarsCarIniLine(slot);
    string filename (rbrdir + "\\cars\\cars.ini");
    for (int i = 0; i < 4; i++)
    {
        string line = freads (filename, replaceLine + i);
        // remove \"\n
        line.resize (line.length() - 2);
        // find filename (assuming 2 subdirs are used)
        int col = line.find("\"") + 1;
        col = line.find("\\", col) + 1;
        col = line.find("\\", col) + 1;
        line = line.substr(col);
        car.setPathsLine(i, line);
    }
}
void readMiscCarinfoIni (RBRCar & car, int slot)
{
    int replaceLine = locateMiscCarinfoIniIniLine(slot);
    string filename (rbrdir + "\\misc\\carinfo.ini");
    for (int i = 0; i < 11; i++)
    {
        string line = freads (filename, replaceLine + i);
        line.resize (line.length() - 1);
        car.setTechDataLine (i, line);
    }
}

void readMiscAiAidrvIniTeam (RBRCar & car, int slot)
{
    string filename (rbrdir + "\\misc\\ai\\aidrv.ini");
    int replaceLine = locateMiscAiAidrvIniTeamLine(slot);
    if (replaceLine == -5) return;
    for (int i = 0; i < 12; i++)
    {
        string line = freads (filename, replaceLine + i);
        line.resize (line.length() - 1);
        car.setTeamDataLine(i, line);        
    }
}

void readMiscAiAidrvIniCar (RBRCar & car, int slot)
{
    string filename (rbrdir + "\\misc\\ai\\aidrv.ini");
    int replaceLine = locateMiscAiAidrvIniCarLine(slot) + 1;
    for (int i = 0; i < 4; i++)
    {
        string line = freads (filename, replaceLine + i);
        line.resize (line.length() - 1);
        car.setCarDataLine(i, line);        
    }
}

int getLength(int slot, char group)
{
    // slots are stored in reverse order in disk
    slot = 7 - slot;
    
    // lengths for every car name. groupA names are equal length than groupA, only with '\0' chars in between.
    int diffA[8]={19, 14, 19, 11, 25, 16, 14, 13};
    int diffC[8]={10,  6, 10,  7, 10,  2,  7,  7};
    
    // compute real final offset by adding the necessary values (depending on group and slot)
    if (group == 'A') return diffA[slot];
    if (group == 'B') return diffA[slot];
    if (group == 'C') return diffC[slot];
}    

void readCarExes (RBRCar & car, int slot)
{
    cout << endl << "[" << task++ << "/" << tasks << "] Reading RichardBurnsRally_SSE.exe..." << endl;
    string exe = rbrdir + "/RichardBurnsRally_SSE.exe";
    
    string nameA = readBinary (exe, getOffset(slot, 'A', true), getLength(slot, 'A'));
    string nameC = readBinary (exe, getOffset(slot, 'C', true), getLength(slot, 'C'));
    
    cout << endl << "[" << task++ << "/" << tasks << "] Reading RichardBurnsRally_NoSSE.exe..." << endl;
    exe = rbrdir + "/RichardBurnsRally_NoSSE.exe";
    string nameAn = readBinary (exe, getOffset(slot, 'A', false), getLength(slot, 'A'));
    string nameCn = readBinary (exe, getOffset(slot, 'C', false), getLength(slot, 'C'));

    if (nameA != nameAn || nameC != nameCn)
    {
        cout << "#### Warning!! The car names of the two .exe files are not equal! ####" << endl;   
        cout << "Using \"RichardBurnsRally_SSE.exe\" version names..." << endl;
    }    
    car.setName(nameC);
    car.setName(nameA);
}    

void importSplashImage (string carDir, int slot)
{
    string splashDir = "splash.tmp\\L#\\Rally_7\\TargetMedia_Develop\\Pc\\Debug\\World\\Textures\\Splash";
    char slotchar[2] = {slot + '0', '\0'};
    string newFilename("car_");
    newFilename.append(slotchar);
    newFilename.append("_1024.dds");

    cout << endl << "[" << task++ << "/" << tasks << "] Extracting \"textures\\splash.dat\" files..." << endl;
    string cmd = "\"" + dattooldir + "\" -e \"" + rbrdir + "\\textures\\splash.dat\" splash.tmp";
    system(cmd.c_str());

    cout << endl << "[" << task++ << "/" << tasks << "] Renaming splash image..." << endl;
    cmd = "ren \"" + splashDir + "\\" + newFilename + "\" splashImage.dds";
    system(cmd.c_str());

    cout << endl << "[" << task++ << "/" << tasks << "] Copying splash image..." << endl;
    cmd = "xcopy /c /h /r /y \"" + splashDir + "\\splashImage.dds\" \"" + carDir + "\"";
    system(cmd.c_str());

    cout << endl << "[" << task++ << "/" << tasks << "] Removing temporal splash.tmp directory..." << endl;
    cleanDir ("splash.tmp");
}   

void readPatchedFiles (RBRCar & car, int slot)
{
    cout << endl << "[" << task++ << "/" << tasks << "] Reading cars.ini file data..." << endl;
    readCarsCarIni (car, slot);

    cout << endl << "[" << task++ << "/" << tasks << "] Reading carinfo.ini file data..." << endl;
    readMiscCarinfoIni (car, slot);

    cout << endl << "[" << task++ << "/" << tasks << "] Reading misc/ai/aidrv.ini teams data..." << endl;
    readMiscAiAidrvIniTeam (car, slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Reading misc/ai/aidrv.ini cars data..." << endl;
    readMiscAiAidrvIniCar (car, slot);
    readCarExes (car, slot);
}    

void importSlot (string slotName, string carDir)
{
    int slot = stoslot (slotName);
    tasks = 18;
    task=1;
    cout << "===============================================================================" << endl;
    cout << "Importing slot #" << slot << " to car dir \"" << carDir << "\"." << endl;    
    cout << "===============================================================================" << endl;

    cout << endl << "[" << task++ << "/" << tasks << "] Creating empty car.ini instance in memory..." << endl;
    RBRCar car;

    cout << endl << "[" << task++ << "/" << tasks << "] Removing existing car directory..." << endl;
    cleanDir ("cars\\" + carDir);
    
    cout << endl << "[" << task++ << "/" << tasks << "] Creating car directory..." << endl;
    createDir ("cars\\" + carDir);
    
    cout << endl << "[" << task++ << "/" << tasks << "] Importing cars graphics data..." << endl;
    importGraphics (slot, carDir);

    cout << endl << "[" << task++ << "/" << tasks << "] Importing physics data..." << endl;
    importPhysics (slot, carDir);

    cout << endl << "[" << task++ << "/" << tasks << "] Reading RBR files modifications to memory..." << endl;
    readPatchedFiles (car, slot);

    cout << endl << "[" << task++ << "/" << tasks << "] Importing RBR files modifications to car.ini..." << endl;
    car.writeIni("cars\\" + carDir + "\\car.ini");

    cout << endl << "[" << task++ << "/" << tasks << "] Importing car splash image..." << endl;
    importSplashImage ("cars\\" + carDir, slot);

    cout << "===============================================================================" << endl;
    cout << "DONE! Slot #" << slot << " succesfully imported to car dir \"" << carDir << "\" :)" << endl;
    cout << "===============================================================================" << endl;
}    



void exportCar (string carDir, string slotName)
{
    int slot = stoslot (slotName);
    tasks = 17;
    task=1;
    cout << "===============================================================================" << endl;
    cout << "Exporting car dir \"" << carDir << "\" to slot #" << slot << "." << endl;    
    cout << "===============================================================================" << endl;

    cout << endl << "[" << task++ << "/" << tasks << "] Loading car.ini file..." << endl;
    RBRCar car (carDir);
    cout << endl << "[" << task++ << "/" << tasks << "] Removing old slot dir..." << endl;
    cleanSlot (slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Exporting cars dir to new slot dir..." << endl;
    exportGraphics (car.getCarDir(), slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Exporting physics to new physics dir..." << endl;
    exportPhysics (car.getCarDir(), slot);
    patchFiles (car, slot);
    
    cout << "===============================================================================" << endl;
    cout << endl << endl << "DONE! Car \"" << carDir << "\" succesfully exported to slot #" << slot << " :)" << endl;
    cout << "===============================================================================" << endl;
}

string getGraphicsSlotDir (int slot)
{
    //return slot# theorical dir, rbr/cars/slotXX dir.
    return (rbrdir + "\\cars\\slot" + getSlotNumber(slot));
}
string getPhysicsSlotDir (int slot)
{
    string slotDir;
    switch (slot)
    {
    case 0:
        slotDir = "c_xsara";
        break;
    case 1:
        slotDir = "h_accent";
        break;
    case 2:
        slotDir = "mg_zr";
        break;
    case 3:
        slotDir = "m_lancer";
        break;
    case 4:
        slotDir = "p_206";
        break;
    case 5:
        slotDir = "s_i2003";
        break;
    case 6:
        slotDir = "t_coroll";
        break;
    case 7:
        slotDir = "s_i2000";
        break;
    default: break;
    }    
    slotDir = (rbrdir + "\\physics\\" + slotDir);
    return slotDir;
}    

void exportGraphics (string car, int slot)
{
    string slotDir = getGraphicsSlotDir(slot);
    string tmpString ("xcopy /e /c /h /r /y /i /q \"" + car + "\\graphics\" \"" + slotDir + "\"");
    system (tmpString.c_str());
}    
void exportPhysics (string car, int slot)
{
    string slotDir = getPhysicsSlotDir(slot);
    string tmpString ("xcopy /e /c /h /r /y /i /q \"" + car + "\\physics\" \"" + slotDir + "\"");
    system (tmpString.c_str());
}    


void cleanSlot (int slot)
{
    // remove slot directory
    cleanDir (getGraphicsSlotDir(slot));
    cleanDir (getPhysicsSlotDir(slot));
}    

void patchFiles (RBRCar car, int slot)
{
    cout << endl << "[" << task++ << "/" << tasks << "] Patching cars.ini file..." << endl;
    patchCarsCarIni (car, slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Patching carinfo.ini file..." << endl;
    patchMiscCarinfoIni (car, slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Patching car shaders .ini file..." << endl;
    patchShadersIni (car, slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Patching misc/ai/aidrv.ini teams..." << endl;
    patchMiscAiAidrvIniTeam (car, slot);
    cout << endl << "[" << task++ << "/" << tasks << "] Patching misc/ai/aidrv.ini cars..." << endl;
    patchMiscAiAidrvIniCar (car, slot);
    patchCarExes (car, slot);
    patchSplashImage (car, slot);    
}

void patchSplashImage (RBRCar car, int slot)
{
    string splashDir = "splash.tmp\\L#\\Rally_7\\TargetMedia_Develop\\Pc\\Debug\\World\\Textures\\Splash";
    char slotchar[2] = {slot + '0', '\0'};
    string newFilename("car_");
    newFilename.append(slotchar);
    newFilename.append("_1024.dds");

    cout << endl << "[" << task++ << "/" << tasks << "] Extracting \"textures\\splash.dat\" files..." << endl;
    string cmd = "\"" + dattooldir + "\" -e \"" + rbrdir + "\\textures\\splash.dat\" splash.tmp";
    system(cmd.c_str());
    
    cout << endl << "[" << task++ << "/" << tasks << "] Copying splash image..." << endl;
    cmd = "xcopy /c /h /r /y \"cars\\" + car.getCar() + "\\splashImage.dds\" \"" + splashDir + "\\\"";
    system(cmd.c_str());

    cout << endl << "[" << task++ << "/" << tasks << "] Removing old splash image..." << endl;
    cmd = "del \"" + splashDir + "\\" + newFilename + "\"";
    system(cmd.c_str());
    
    cout << endl << "[" << task++ << "/" << tasks << "] Renaming new splash image..." << endl;
    cmd = "ren \"" + splashDir + "\\splashImage.dds\" \"" + newFilename;
    system(cmd.c_str());
    
    cout << endl << "[" << task++ << "/" << tasks << "] Repacking \"textures\\splash.dat\" file..." << endl;    
    cmd = "\"" + dattooldir + "\" -c splash.tmp \"" + rbrdir + "\\textures\\splash.dat\"";
    system(cmd.c_str());
    
    cout << endl << "[" << task++ << "/" << tasks << "] Removing temporal splash.tmp directory..." << endl;
    cleanDir ("splash.tmp");
}    

void patchMiscAiAidrvIniCar (RBRCar car, int slot)
{
    string filename (rbrdir + "\\misc\\ai\\aidrv.ini");
    int replaceLine = locateMiscAiAidrvIniCarLine(slot);
    for (int i = 0; i < 4; i++)
    {
        freplaces (filename, replaceLine + i, car.getCarDataLine(i).c_str());
    }
}
int locateMiscAiAidrvIniCarLine(int slot)
{
    FILE * file = fileOpen(rbrdir + "\\misc\\ai\\aidrv.ini", "r");
    
    int nline = 1, carCount = -1;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring = tmpstring.substr(0, tmpstring.size() -1);
        if (tmpstring == "; Cars")
        {
            carCount = 0;
        }
        if (carCount != -1 && tmpstring == "; Name on own line")
        {
            if (carCount == slot) return nline + 1;
            carCount++;
        }    
        nline++;
    }
    fclose (file);
    return -1;
}


void patchMiscAiAidrvIniTeam (RBRCar car, int slot)
{
    string filename (rbrdir + "\\misc\\ai\\aidrv.ini");
    int replaceLine = locateMiscAiAidrvIniTeamLine(slot);
    if (replaceLine == -5) return;
    for (int i = 0; i < 12; i++)
    {
        freplaces (filename, replaceLine + i, car.getTeamDataLine(i).c_str());
    }
}
int locateMiscAiAidrvIniTeamLine(int slot)
{
    if (slot == 2) return -5;
    if (slot > 2) slot--;
    FILE * file = fileOpen(rbrdir + "\\misc\\ai\\aidrv.ini", "r");
    
    int nline = 1, teamCount = -1;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring = tmpstring.substr(0, tmpstring.size() -1);
        if (tmpstring == "; Teams")
        {
            teamCount = 0;
        }
        if (teamCount != -1 && tmpstring == "; Name, on own line")
        {
            if (teamCount == slot) return nline + 1;
            teamCount++;
        }
        nline++;
    }
    fclose (file);
    return -1;
}

int getOffset(int slot, char group, bool sse)
{
    //find out actual byte where the name is in the .exe
    
    // slots are stored in reverse order in disk
    slot = 7 - slot;
    
    // offset bytes for the sse.exe and nosse.exe files
    int AoffsetSSE   = 3345868;
    int AoffsetNoSSE = 3293564;
    // offsets to group B and C of names.
    int Boffset = 172;
    int Coffset = 172 + 284;

    // offset differences for every car name (slot), in every of the 3 groups
    int diffA[8]={0, 20, 16, 20, 12, 28, 20, 16};
    int diffB[8]={0, 40, 32, 40, 24, 52, 36, 32};
    int diffC[8]={0, 12,  8, 12,  8, 12,  4,  8};
    
    // compute actual offset by using the differences
    for (int i=1; i<8; i++)
    {
        diffA[i] += diffA[i-1];
        diffB[i] += diffB[i-1];        
        diffC[i] += diffC[i-1];
    }    
    
    // compute real final offset by adding the necessary values (depending on group and slot)
    int finalOffset = sse ? AoffsetSSE : AoffsetNoSSE;
    if (group == 'A') finalOffset += 0       + diffA[slot];
    if (group == 'B') finalOffset += Boffset + diffB[slot];
    if (group == 'C') finalOffset += Coffset + diffC[slot];
    
    // return the real address (# of bytes) in the exe file.
    return finalOffset;
}    


void patchCarExes (RBRCar car, int slot)
{
    string nameA = car.getBestNameA(slot);
    string nameB = car.getBestNameA(slot);
    string nameC = car.getBestNameC(slot);

    cout << endl << "[" << task++ << "/" << tasks << "] Patching RichardBurnsRally_SSE.exe..." << endl;
    string exe = rbrdir + "/RichardBurnsRally_SSE.exe";
    patchBinary (exe, getOffset(slot, 'A', true), nameA);
    patchBinaryB (exe, getOffset(slot, 'B', true), nameB);
    patchBinary (exe, getOffset(slot, 'C', true), nameC);
    
    cout << endl << "[" << task++ << "/" << tasks << "] Patching RichardBurnsRally_NoSSE.exe..." << endl;
    exe = rbrdir + "/RichardBurnsRally_NoSSE.exe";
    patchBinary (exe, getOffset(slot, 'A', false), nameA);
    patchBinaryB (exe, getOffset(slot, 'B', false), nameB);
    patchBinary (exe, getOffset(slot, 'C', false), nameC);
}    
void patchShadersIni (RBRCar car, int slot)
{
    string filename (rbrdir + "\\" + car.getShaderPath(slot));
    int replaceLine = locateShaderIniLine(filename);
    for (int i = 0; i < 10; i++)
    {
        freplaces (filename, replaceLine + i, car.getShaderLine(i, slot).c_str());
    }
}
int locateShaderIniLine(string shaderFile)
{
    FILE * file = fileOpen(shaderFile, "r");
    
    int nline = 1;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring.resize (tmpstring.length() -1);
        
        if (tmpstring == "[TSDefault]")
        {
            return nline + 1;
        }
        nline++;
    }
    fclose (file);
    return -1;
}
void patchMiscCarinfoIni (RBRCar car, int slot)
{
    int replaceLine = locateMiscCarinfoIniIniLine(slot);
    string filename (rbrdir + "\\misc\\carinfo.ini");
    freplaces(filename, replaceLine - 1, car.getTechDataHeader(slot));
    for (int i = 0; i < 11; i++)
    {
        freplaces (filename, replaceLine + i, car.getTechDataLine(i).c_str());
    }
}
void patchCarsCarIni (RBRCar car, int slot)
{
    int replaceLine = locateCarsCarIniLine(slot);
    string filename (rbrdir + "\\cars\\cars.ini");
    for (int i = 0; i < 5; i++)
    {
        freplaces (filename, replaceLine + i, car.getPathLine(i, slot).c_str());
    }
}    

int locateMiscCarinfoIniIniLine (int slot)
{
    FILE * file = fileOpen(rbrdir + "\\misc\\carinfo.ini", "r");
    
    int nline = 1;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    string slotSection ("; Car" + getSlotNumber(slot) + " (");
    slotSection[5]=' ';
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring = tmpstring.substr(0, 9);
        if (tmpstring == slotSection)
        {
            return nline + 1;
        }
        nline++;
    }
    fclose (file);
    return -1;
}    

int locateCarsCarIniLine (int slot)
{
    FILE * file = fileOpen(rbrdir + "\\cars\\cars.ini", "r");
    
    int nline = 1;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    string slotSection ("[Car" + getSlotNumber(slot) + "]");
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring.resize (tmpstring.length() -1);
        
        if (tmpstring == slotSection)
        {
            return nline + 1;
        }
        nline++;
    }
    fclose (file);
    return -1;
}    


