#include "rbrcar.hpp"

void RBRCar::setName(string name)
{
    int lengths[10] = {25, 19, 16, 14, 13, 11, 10, 7, 6, 2};
    for (int i = 0; i < 10; i++)
    {
        if (name.length() <= lengths[i]) names[i] = name;
    }    
}    
void RBRCar::setPathsLine(int n, string line)
{
    paths[n] = line;
}    
void RBRCar::setTechDataLine(int n, string line)
{
    techData[n] = line;
}    
void RBRCar::setTeamDataLine(int n, string line)
{
    teamData[n] = line;
}    
void RBRCar::setCarDataLine(int n, string line)
{
    carData[n] = line;
}    

void RBRCar::writeIni(string filename)
{
    FILE * file = fileOpen(filename, "w");
    string tmp = "[RBRCarManager V";
    tmp.append (version + "]\n");
    fputs(tmp.c_str(), file);
    int i;
    fputs("\n[car names]\n", file);
    for (i = 0; i < 10; i++)
    {
        fputs(names[i].c_str(), file);
        fputs("\n", file);
    }        
    fputs("\n[./cars/cars.ini data]\n", file);
    for (i = 0; i < 4; i++)
    {
        fputs(paths[i].c_str(), file);
        fputs("\n", file);
    }
    fputs("\n[./misc/carinfo.ini car tech data]\n", file);
    for (i = 0; i < 11; i++)
    {
        fputs(techData[i].c_str(), file);
        fputs("\n", file);
    }
    fputs("\n[./misc/ai/aidrv.ini team data]\n", file);
    for (i = 0; i < 12; i++)
    {
        fputs(teamData[i].c_str(), file);
        fputs("\n", file);
    }
    fputs("\n[./misc/ai/aidrv.ini car data]\n", file);
    for (i = 0; i < 4; i++)
    {
        fputs(carData[i].c_str(), file);
        fputs("\n", file);
    }
    fclose(file);    
}    


string RBRCar::getBestNameA(int slot)
{
    int index[8]={4, 3, 2, 0, 5, 1, 3, 1};
    return names[index[slot]];
}    

string RBRCar::getBestNameB(int slot)
{
    string name = getBestNameA(slot);
    int finalLength = name.length() * 2;
    
    for (int i = 0; i < finalLength; i+=2)
    {
        name.insert(i, 1, 0);
    }    
}    

string RBRCar::getBestNameC(int slot) ///////fixmeeeeeee
{
    int index[8]={7, 7, 9, 6, 7, 6, 8, 6};
    return names[index[slot]];
}    

string RBRCar::getTeamDataLine (int n)
{
    return teamData[n];   
}    
string RBRCar::getCarDataLine (int n)
{
    return carData[n];   
}    

string RBRCar::getTechDataLine (int n)
{
    return techData[n];
}    
string RBRCar::getTechDataHeader(int slot)
{
    string header ("; Car" + getSlotNumber(slot) + " (" + names[0] + ")");
    header[5]=' ';
    return header;
}    

string RBRCar::getPathLine(int n, int slot)
{
    if (n == 0) return ("FileName        = \"Cars\\slot" + getSlotNumber(slot) + "\\" + paths[n] + "\"");
    if (n == 1) return ("IniFile         = \"Cars\\slot" + getSlotNumber(slot) + "\\" + paths[n] + "\"");    
    if (n == 2) return ("ShaderFile      = \"Cars\\slot" + getSlotNumber(slot) + "\\" + paths[n] + "\"");
    if (n == 3) return ("ShaderSettings  = \"Cars\\slot" + getSlotNumber(slot) + "\\" + paths[n] + "\"");
    if (n == 4) return ("TexturePath     = \"Cars\\slot" + getSlotNumber(slot) + "\\Textures\\\"");
    return ("RBRCM error!");
}    

string RBRCar::getShaderLine(int n, int slot)
{
    int initialPosition = shader[n].find("\\");
    //fix for the MG_ZR, which lacks a shader line (it has only 9 instead of 10).
    if (initialPosition == string::npos)
    {
        return (shader[n]);
    }else{
        // move the positioner 1 chars ("=") to the right.
        initialPosition += 1;
    }
    int finalPosition = shader[n].find("\\textures\\");
    string line (shader[n].substr(0, initialPosition));
    line.append("slot" + getSlotNumber(slot) + shader[n].substr(finalPosition));
    return (line);
}    
string RBRCar::getShaderPath(int slot)
{
    return ("Cars\\slot" + getSlotNumber(slot) + "\\" + paths[2]);
}    
string RBRCar::getCar ()
{
    return (car);
}    
string RBRCar::getCarDir ()
{
    return ("cars\\" + car);
}    
void RBRCar::clean ()
{
    int i;
    for (i = 0; i < 10; i++) names[i] = "";
    for (i = 0; i < 4; i++) paths[i] = "";
    for (i = 0; i < 11; i++) techData[i] = "";
    for (i = 0; i < 12; i++) teamData[i] = "";
    for (i = 0; i < 4; i++) carData[i] = "";
}    

void RBRCar::print ()
{
    int i;
    cout << "Names:" << endl;
    for (i = 0; i < 10; i++) cout << names[i] << endl;   
    cout << "Paths:" << endl;
    for (i = 0; i < 4; i++) cout << paths[i] << endl;   
    cout << "Shaders:" << endl;
    for (i = 0; i < 10; i++) cout << shader[i] << endl;   
    cout << "Tech Data:" << endl;
    for (i = 0; i < 11; i++) cout << techData[i] << endl;   
    cout << "Team Data:" << endl;
    for (i = 0; i < 12; i++) cout << teamData[i] << endl;   
    cout << "Car Data:" << endl;
    for (i = 0; i < 4; i++) cout << carData[i] << endl;   
}
RBRCar::RBRCar (void)
{
    clean();
}
RBRCar::RBRCar (string carDir)
{
    // open car.ini file in read mode
    string iniPath = "cars/" + carDir + "/car.ini";
    car = carDir;
    FILE * file = fileOpen(iniPath, "r");

    // set all properties to zero before parsing    
    clean();
    
    // initialize parsing loop
    int nline = 1, sectionLine = 0;
    SectionType sectionType = NONE;
    const int linesize = 4096;
    char tmpline[linesize];
    
    if (fgets(tmpline, linesize, file))
    {
        string tmpstring (tmpline);
        tmpstring.resize (tmpstring.length() -1);
        if (tmpstring != "[RBRCarManager V" + version + "]")
        {
            cout << "#### Warning!! This RBRCM car files format is not RBRCM v" << version << " compliant! ####" << endl;
        }
        nline++;
    }    
    
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring.resize (tmpstring.length() -1);
        switch (sectionType)
        {
        case NAMES:
            if (sectionLine >= 10) sectionType = NONE;
            else names[sectionLine] = tmpstring;
            break;
        case PATHS:
            if (sectionLine >= 4) sectionType = NONE;
            else paths[sectionLine] = tmpstring;
            break;
        case TECHDATA:
            if (sectionLine >= 11) sectionType = NONE;
            else techData[sectionLine] = tmpstring;
            break;
        case TEAMDATA:
            if (sectionLine >= 12) sectionType = NONE;
            else teamData[sectionLine] = tmpstring;
            break;
        case CARDATA:
            if (sectionLine >= 4) sectionType = NONE;
            else carData[sectionLine] = tmpstring;
            break;
        default:
            if (tmpstring[0] == '[')
            {
                sectionType = identifySectionType (tmpstring);
                sectionLine = -1;
            }
            break;
        }
        nline++;
        sectionLine++;
    }
    fclose (file);
    
    // pad the names with blank spaces where possible
    int size[10] = {25, 19, 16, 14, 13, 11, 10, 7, 6, 2};
    for (int i = 0; i < 10; i++) names[i].append( size[i] - names[i].size(), ' ');

    // parse the shaders file harcoded texture paths    
    iniPath = "cars/" + carDir + "/graphics/" + paths[2];
    file = fileOpen(iniPath, "r");
    nline = 1;
    int parsedLine = 0;
    bool sectionFound = false;
    while ( fgets (tmpline, linesize, file) )
    {
        string tmpstring (tmpline);
        tmpstring.resize (tmpstring.length() -1);
        
        if (sectionFound && parsedLine < 10)
        {
            shader[parsedLine] = tmpstring;
            parsedLine++;
        }    
        if (tmpstring == "[TSDefault]")
        {
            sectionFound = true;
            parsedLine = 0;
        }
        nline++;
    }
    fclose (file);
    
    cout << "Car \"" << names[0] << "\" in memory." << endl;
}    

